@@ -18,9 +18,9 @@
 #include <SDL/SDL.h>
 
 #include "../main.h"
-#include "../network.h"
-#include "../tileengine.h"
-#include "../unicode2ascii.h"
+#include "../io/network.h"
+#include "../tile_engine/tileengine.h"
+#include "../display/unicode2ascii.h"
 
 #include "coordinate_reader.h"
 
