@@ -21,7 +21,7 @@
 
 #include "../main.h"
 
-char* google_prepare_url(char* encodedQuery);
+char* google_prepare_url(void* encodedQuery);
 void google_parse_response(char* response);
 
 #endif /* GOOGLE_GEOCODER_H_ */
