@@ -24,6 +24,6 @@
 #define GOOGLE_QUERY_URL "http://maps.googleapis.com/maps/api/geocode/json?address=%s&sensor=true"
 
 char* google_local_prepare_url(char* encodedQuery);
-void google_local_parse_response(char* response);
+void google_local_parse_response(void* response);
 
 #endif /* GOOGLE_LOCAL_GEOCODER_H_ */
