@@ -18,15 +18,15 @@
 #include <SDL/SDL.h>
 #include <math.h>
 
-#if defined(N900) || defined N950
-#include <GLES/gl.h>
-#include <GLES/glext.h>
-#else
-#include <GL/gl.h>
-#endif
+
+#include "gl.h"
+#include "menu/Menu.h"
+#include "vkb/VirtualKeyboard.h"
+#include "searchbar/SearchBar.h"
+#include "about/AboutDialog.h"
 
 #define M_PI 3.14159265358979323846
-#define CLOUDGPS_VERION "0.5.5"
+#define CLOUDGPS_VERSION "0.6.4"
 
 /** Equatorial earth radius in meters. */
 #define EARTH_RADIUS 6378137
@@ -38,12 +38,19 @@
 #endif
 #define SCREEN_HEIGHT 480
 #define TILE_SIZE     256
-#define PORTRAIT_ROTATION_Y 14
+#define PORTRAIT_ROTATION_Y 12
 #define LANDSCAPE_ROTATION_X 14
 
 // how many satellite bars are drawn in status bar
 #define MAX_SATTELITES 12
 
+#ifndef N950
+#define MAX_MOUSE_POINTERS 2
+#else
+#define MAX_MOUSE_POINTERS SDL_MAXMOUSE
+#endif
+
+
 // how many tiles horizontally and vertically
 #ifdef N900
 #define TILES_X 6
@@ -53,6 +60,7 @@
 #define TILES_Y 8
 #endif
 
+void setQuitFlag();
 
 #define MAX_TEXTURES_CREATED_PER_FRAME 1
 
@@ -72,6 +80,7 @@ typedef struct {
 
 extern GList* geocodingProviders;
 extern GList* routingProviders;
+extern GList* tileProviders;
 
 typedef struct {
     double latitude, longitude;
@@ -91,9 +100,9 @@ typedef struct {
 } Color;
 
 typedef struct {
-   const char* name;
-   const char* description;
-   const char* wikipedia;
+   char* name;
+   char* description;
+   char* wikipedia;
    WorldCoordinate centroid;
    WorldCoordinate bounds[2];
 } GeoCodeResult;
@@ -112,7 +121,7 @@ typedef struct {
 	int raiseLevel;
 } ScreenMarker;
 
-struct {
+typedef struct {
     int mipmapToggle;
     int showGrid, showCoordinates, accelerometerEnabled;
     int showDebugTiles;
@@ -126,17 +135,20 @@ struct {
 
     // TODO use them to determine behaviour on tile rendering
     int useFading, useTileSync;
-
+    bool useVirtualKeyboard;
     Orientation orientation;
     char *userAgent;
     char *referer;
+    char *ttfFont;
     BackgroundQueryProvider* geocodingProvider;
     BackgroundQueryProvider* routingProvider;
     int snapToRouteToleranceMeters;
     int outsideRouteToleranceMeters;
     int recalculateRouteAfterOutSideRouteSeconds;
-} options;
+    int hasCompass;
+} Options;
 
+extern Options options;
 //TTF_Font *font = NULL;
 
 typedef struct {
@@ -152,6 +164,10 @@ typedef enum {
     VIEW_2D, VIEW_3D
 } ViewMode;
 
+typedef enum {
+    ROTATION_ALWAYS_NORTH = 0, ROTATION_FREE = 1, ROTATION_USING_COMPASS = 2
+} RotationMode;
+
 typedef enum {
     UI_HIDDEN = 0, UI_COMING = 1, UI_SHOWN = 2, UI_GOING = 3
 } uiElemStatus;
@@ -234,10 +250,31 @@ typedef struct {
 	RouteDirection* currentDirection;
 	double distanceToTarget;
 	int roundedDistanceToNextDirection;
-	int currentDirectionIdx;
+	GLuint currentDirectionIdx;
 	int currentLineIdx;
 } NavigationStatus;
 
+typedef struct {
+	int prn;
+	int signalStrength;
+	int inUse;
+	double elevation;
+	double azimuth;
+} SatelliteInfo;
+
+typedef struct {
+	int positionKnown;
+	int hasGpsFix;
+	WorldCoordinate position;
+	double groundSpeed;
+	double altitude;
+	double heading;
+	double horizontalAccuracy;
+	SatelliteInfo satellites[MAX_SATTELITES];
+	int satellitesInView;
+	int satellitesInUse;
+} CurrentPositionInfo;
+
 typedef struct {
     GLfloat x, y, dx, dy, scale, oldScale, destScale, rotx, roty, rotz, destRotx, destRoty, destRotz, drotz, friction;
     int attractedToRotZ;
@@ -247,9 +284,9 @@ typedef struct {
     WorldCoordinate center;
     WorldCoordinate previousCenter; // before following mode turned on
     WorldCoordinate attraction; // position to which canvas is attracted (ie current position in following mode)
-    WorldCoordinate currentPos;
     TileProvider* provider;
     ViewMode viewMode;
+    CurrentPositionInfo currentPositionInfo;
     GLfloat orientationTransition;
     int orientationTransitionLinear;
     int followingMypos;
@@ -257,7 +294,7 @@ typedef struct {
     int routeFlybyMode;
     int routeFlybyLineIdx;
 
-    int rotation2d;
+    int rotation;
     int searchBarActive;
     int zoomBarActive;
     long zoomBarVisibleMilis;
@@ -286,37 +323,38 @@ typedef struct {
     int mouseHoldInPlace;
     WorldCoordinate mouseHoldPositionWc;
     Point mouseHoldPositionSc;
-    PopupMenu popup;
+    Menu* popup;
+    WorldCoordinate popupWc;
+    VirtualKeyboard* virtualKeyboard;
 
     NavigationStatus navigationStatus;
+    bool dimmed;
+    GLfloat dimColor;
+	Menu* mainMenu;
+	Menu* optionsMenu;
+	Menu* mapChooseMenu;
+	Menu* searchServiceMenu;
+	Menu* routingServiceMenu;
+	Menu* keyboardLanguageMenu;
+	AboutDialog* aboutDialog;
+	SearchBar* searchBar;
 } Canvas;
+
 extern Canvas canvas;
 
-#define SEARCHBAR_MAX_CHARS 100
-
-typedef struct {
-    char input[SEARCHBAR_MAX_CHARS];
-    int cursorScreen; // Cursor position on screen (in pixels)
-    int cursorInput; // Cursor position in input string (in bytes)
-    int selectionStart;
-    int selectionCount;
-    int historyIndex;
-    char** history;
-    long lastChange;
-} SearchBar;
-extern SearchBar searchBar;
-
 typedef struct {
     int x, y, oldx, oldy, button, oldButton, xdelta, ydelta;
     int clickedx, clickedy;
     int draggedDistanceX, draggedDistanceY;
     long pressed, moved; // milliseconds of last pressed and moved events, -1 if mousebutton is not down
+    bool processed;
 } Mouse;
-extern Mouse mouse;
+
+extern Mouse mouse[MAX_MOUSE_POINTERS];
 
 typedef struct {
-    int x, y, z;
-    int calibrateX, calibrateY, calibrateZ;
+    float x, y, z;
+    float calibrateX, calibrateY, calibrateZ;
     int performCalibration; // if set to TRUE calibration will be performed on next accelerometer read.
 } Accelerometer;
 extern Accelerometer accelerometer;
@@ -334,9 +372,4 @@ extern long nowMillis;
 extern GQueue* downloadQueue;
 extern int tileCoordinateVisibility[TILES_X + 1][TILES_Y + 1];
 
-#ifdef N900
-#include <location/location-gps-device.h>
-extern LocationGPSDevice *device;
-#endif
-
 #endif /* MAIN_H_ */
